core:import("CoreControllerWrapperSettings")

_G.EATDKeybinds = _G.EATDKeybinds or {}
local Keybinds = Keybinds or _G.EATDKeybinds

Keybinds._keybinds = {}

Keybinds._option_categories = {
	"normal",
	"vehicle"
}

function Keybinds:add_category( id )
	table.insert( Keybinds._option_categories, id )
end

function Keybinds:add_button( id, category )
	local controller_manager = managers.controller
	local settings_map = controller_manager:get_settings_map()

	local data = {
		name = id,
		inputs = {}
	}

	settings_map.pc:set_connection( id, CoreControllerWrapperSettings.ControllerWrapperButton:new(data) )

	Keybinds._keybinds[id] = data

	table.insert( MenuCustomizeControllerCreator.CONTROLS, id )

	MenuCustomizeControllerCreator.CONTROLS_INFO[id] = {
		category = category or "modded",
		text_id = "menu_button_" .. id
	}
end

function MenuCustomizeControllerCreator:setup_node(node)
	local new_node = node
	local controller_category = node:parameters().controller_category or "normal"

	node:clean_items()

	local params = {
		callback = "choice_controller_type",
		name = "controller_type",
		text_id = "menu_controller_type"
	}
	local data_node = {type = "MenuItemMultiChoice"}

	for _, category in pairs(Keybinds._option_categories) do
		table.insert(data_node, {
			value = category,
			text_id = "menu_controller_" .. category,
			_meta = "option"
		})
	end

	local new_item = node:create_item(data_node, params)

	new_item:set_value(controller_category)
	node:add_item(new_item)

	local controller_manager = managers.controller
	local settings_map = controller_manager:get_settings_map()

	local connections = managers.controller:get_settings(managers.controller:get_default_wrapper_type()):get_connection_map()

	for _, name in ipairs(self.controls_info_by_category(controller_category)) do
		local name_id = name
		local connection = connections[name]

		if not connection then
			local data = Keybinds._keybinds[name]
			settings_map.pc:set_connection( name, CoreControllerWrapperSettings.ControllerWrapperButton:new(data) )

			connections = managers.controller:get_settings(managers.controller:get_default_wrapper_type()):get_connection_map()
			connection = connections[name]
		end

		if connection._btn_connections then
			local ordered = self.AXIS_ORDERED[name]

			for _, btn_name in ipairs(ordered) do
				local btn_connection = connection._btn_connections[btn_name]

				if btn_connection then
					local name_id = name
					local params = {
						localize = "false",
						name = btn_name,
						connection_name = name,
						text_id = utf8.to_upper(managers.localization:text(self.CONTROLS_INFO[btn_name].text_id)),
						binding = btn_connection.name,
						axis = connection._name,
						button = btn_name
					}
					local new_item = new_node:create_item({type = "MenuItemCustomizeController"}, params)

					new_node:add_item(new_item)
				end
			end
		else
			local params = {
				localize = "false",
				name = name_id,
				connection_name = name,
				text_id = utf8.to_upper(managers.localization:text(self.CONTROLS_INFO[name].text_id)),
				binding = connection:get_input_name_list()[1],
				button = name
			}
			local new_item = new_node:create_item({type = "MenuItemCustomizeController"}, params)

			new_node:add_item(new_item)
		end
	end

	local params = {
		callback = "set_default_controller",
		name = "set_default_controller",
		text_id = "menu_set_default_controller",
		category = controller_category
	}
	local new_item = new_node:create_item(nil, params)

	new_node:add_item(new_item)
	managers.menu:add_back_button(new_node)

	return new_node
end

Hooks:Add("LocalizationManagerPostInit", "ExtraKeybinds_LocalizationManagerPostInit", function(loc)
	loc:add_localized_strings({
		["menu_controller_eatd"] = "EATD"
	})
end)


Hooks:Add("MenuManagerBuildCustomMenus", "ExtraKeybinds_MenuManagerBuildCustomMenus", function( menu_manager, nodes )
	EATDKeybinds:add_category( "eatd" )
end)