function BlackMarketManager:equipped_bayonet(weapon_id)
	return nil
end

function BlackMarketManager:actual_equipped_bayonet(weapon_id, primary)
	local available_weapon_mods = managers.weapon_factory:get_parts_from_weapon_id(weapon_id)
	local equipped_weapon_mods = managers.blackmarket:equipped_item(primary and "primaries" or "secondaries").blueprint

	if available_weapon_mods and available_weapon_mods.bayonet then
		for _, mod in ipairs(equipped_weapon_mods) do
			if tweak_data.weapon.factory.parts[mod].type == "bayonet" or tweak_data.weapon.factory.parts[mod].sub_type == "bayonet" then
				return mod
			end
		end
	end

	return nil
end

function BlackMarketManager:get_melee_data( bayonet_id, factory_id, blueprint )
	local base_melee_data = tweak_data.blackmarket.melee_weapons.weapon

	local bayonet_data = managers.weapon_factory:get_part_data_by_part_id_from_weapon( bayonet_id, factory_id, blueprint )
	local melee_data = bayonet_data.melee

	local final_melee_data = deep_clone(base_melee_data)
	final_melee_data.name_id = tweak_data.weapon.factory.parts[bayonet_id].name_id or final_melee_data.name_id

	if melee_data then
		for key, attribute in pairs( melee_data ) do
			if type(attribute) == "table" then
				final_melee_data[key] = final_melee_data[key] or {}

				for sub_key, sub_attribute in pairs( attribute ) do
					if type(sub_attribute) == "table" then
						final_melee_data[key][sub_key] = final_melee_data[key][sub_key] or {}

						for sub_sub_key, sub_sub_attribute in pairs( sub_attribute ) do
							final_melee_data[key][sub_key][sub_sub_key] = sub_sub_attribute
						end
					else
						final_melee_data[key][sub_key] = sub_attribute
					end
				end
			else
				final_melee_data[key] = attribute
			end
		end
	end

	return final_melee_data
end

function BlackMarketManager:equipped_melee_weapon_damage_info_normal(lerp_value)
	lerp_value = lerp_value or 0

	local melee_entry = self:equipped_melee_weapon()
	local stats = tweak_data.blackmarket.melee_weapons[melee_entry].stats

	local dmg = math.lerp(stats.min_damage, stats.max_damage, lerp_value)
	local dmg_effect = dmg * math.lerp(stats.min_damage_effect, stats.max_damage_effect, lerp_value)

	return dmg, dmg_effect
end

function BlackMarketManager:equipped_melee_weapon_damage_info_bayonet(lerp_value, bayonet_data)
	lerp_value = lerp_value or 0

	local stats = bayonet_data.stats

	local dmg = math.lerp(stats.min_damage, stats.max_damage, lerp_value)
	local dmg_effect = dmg * math.lerp(stats.min_damage_effect, stats.max_damage_effect, lerp_value)

	return dmg, dmg_effect
end

function BlackMarketManager:equipped_melee_weapon_damage_info(lerp_value)
	local player = managers.player:player_unit()
	local state = player:movement():current_state()

	if state and state.bayonet_id then
		return self:equipped_melee_weapon_damage_info_bayonet( lerp_value, state.melee_data )
	end

	return self:equipped_melee_weapon_damage_info_normal( lerp_value )
end