function PlayerStandard:_check_action_melee(t, input)
	if self._state_data.melee_attack_wanted then
		if not self._state_data.melee_attack_allowed_t then
			self._state_data.melee_attack_wanted = nil

			self:_do_action_melee(t, input)
		end

		return
	end

	local main_bool = self._controller:get_input_bool("melee") or false
	local main_release = self._controller:get_input_released("melee")  or false
	local main_pressed = self._controller:get_input_pressed("melee")  or false

	local alt_bool = self._controller:get_input_bool("eatd_melee_butt") or false
	local alt_release = self._controller:get_input_released("eatd_melee_butt") or false
	local alt_pressed = self._controller:get_input_pressed("eatd_melee_butt") or false

	local main_melee = main_bool or main_release or main_pressed
	local alternate_melee = alt_bool or alt_release or alt_pressed

	if managers.blackmarket:equipped_melee_weapon() == "weapon" then
		local action_wanted = main_pressed or main_release or self._state_data.melee_charge_wanted

		if not action_wanted then
			return
		end

		input.btn_melee_press = main_pressed
		input.btn_melee_release = main_release
		input.btn_meleet_state = main_bool

		self:_check_action_melee_bayonet( t, input )
	else
		local alt_action_wanted = alt_pressed or alt_release or self._state_data.melee_charge_wanted
		if alt_action_wanted and ( self._state_data.alternate_melee or alternate_melee ) then
			input.btn_melee_press = alt_pressed
			input.btn_melee_release = alt_release
			input.btn_meleet_state = alt_bool

			self:_check_action_melee_bayonet( t, input )
		end

		local main_action_wanted = main_pressed or main_release or self._state_data.melee_charge_wanted
		if main_action_wanted and ( self._state_data.normal_melee or main_melee ) then
			input.btn_melee_press = main_pressed
			input.btn_melee_release = main_release
			input.btn_meleet_state = main_bool

			self:_check_action_melee_normal( t, input )
		end
	end
end

function PlayerStandard:_check_action_melee_normal(t, input)
	if self._state_data.alternate_melee then return end

	if input.btn_melee_release then
		if self._state_data.meleeing then
			if self._state_data.melee_attack_allowed_t then
				self._state_data.melee_attack_wanted = true

				return
			end

			self:_do_action_melee(t, input)
		end

		return
	end

	local action_forbidden = not self:_melee_repeat_allowed() or self._use_item_expire_t or self:_changing_weapon() or self:_interacting() or self:_is_throwing_projectile() or self:_is_using_bipod()

	if action_forbidden then
		return
	end

	self._state_data.normal_melee = true

	local melee_entry = managers.blackmarket:equipped_melee_weapon()
	local instant = tweak_data.blackmarket.melee_weapons[melee_entry].instant

	self:_start_action_melee(t, input, instant)

	return true
end

function PlayerStandard:_check_action_melee_bayonet(t, input)
	if self._state_data.normal_melee then return end

	if input.btn_melee_release then
		if self._state_data.meleeing then
			if self._state_data.melee_attack_allowed_t then
				self._state_data.melee_attack_wanted = true

				return
			end

			self:_do_action_melee(t, input)
		end

		return
	end

	local action_forbidden = not self:_melee_repeat_allowed() or self._use_item_expire_t or self:_changing_weapon() or self:_interacting() or self:_is_throwing_projectile() or self:_is_using_bipod()

	if action_forbidden then
		return
	end

	self._state_data.alternate_melee = true

	local instant = self._state_data.melee_data.instant

	self:_start_action_melee(t, input, instant)

	return true
end

function PlayerStandard:_update_melee_timers_normal(t, input)
	if self._state_data.melee_attack_allowed_t and self._state_data.melee_attack_allowed_t <= t then
		self._state_data.melee_start_t = t
		local melee_entry = managers.blackmarket:equipped_melee_weapon()
		local melee_charge_shaker = tweak_data.blackmarket.melee_weapons[melee_entry].melee_charge_shaker or "player_melee_charge"
		self._state_data.melee_charge_shake = self._ext_camera:play_shaker(melee_charge_shaker, 0)
		self._state_data.melee_attack_allowed_t = nil
	end

	if self._state_data.melee_repeat_expire_t and self._state_data.melee_repeat_expire_t <= t then
		self._state_data.melee_repeat_expire_t = nil

		if input.btn_meleet_state then
			local melee_entry = managers.blackmarket:equipped_melee_weapon()
			local instant_hit = tweak_data.blackmarket.melee_weapons[melee_entry].instant
			self._state_data.melee_charge_wanted = not instant_hit and true
		end
	end
end
function PlayerStandard:_update_melee_timers_bayonet(t, input)
	if self._state_data.melee_attack_allowed_t and self._state_data.melee_attack_allowed_t <= t then
		self._state_data.melee_start_t = t
		local melee_charge_shaker = self._state_data.melee_data.melee_charge_shaker or "player_melee_charge"
		self._state_data.melee_charge_shake = self._ext_camera:play_shaker(melee_charge_shaker, 0)
		self._state_data.melee_attack_allowed_t = nil
	end

	if self._state_data.melee_repeat_expire_t and self._state_data.melee_repeat_expire_t <= t then
		self._state_data.melee_repeat_expire_t = nil

		if input.btn_meleet_state then
			local instant_hit = self._state_data.melee_data.instant
			self._state_data.melee_charge_wanted = not instant_hit and true
		end
	end
end

function PlayerStandard:_update_melee_timers(t, input)
	if self._state_data.meleeing then
		local lerp_value = self:_get_melee_charge_lerp_value(t)

		self._camera_unit:anim_state_machine():set_parameter(self:get_animation("melee_charge_state"), "charge_lerp", math.bezier({
			0,
			0,
			1,
			1
		}, lerp_value))

		if self._state_data.melee_charge_shake then
			self._ext_camera:shaker():set_parameter(self._state_data.melee_charge_shake, "amplitude", math.bezier({
				0,
				0,
				1,
				1
			}, lerp_value))
		end
	end

	if self._state_data.melee_damage_delay_t and self._state_data.melee_damage_delay_t <= t then
		self:_do_melee_damage(t, nil, self._state_data.melee_hit_ray)

		self._state_data.melee_damage_delay_t = nil
		self._state_data.melee_hit_ray = nil
	end

	if self._state_data.melee_expire_t and self._state_data.melee_expire_t <= t then
		self._state_data.melee_expire_t = nil
		self._state_data.melee_repeat_expire_t = nil

		self._state_data.alternate_melee = false
		self._state_data.normal_melee = false

		self:_stance_entered()

		if self._equipped_unit and input.btn_steelsight_state then
			self._steelsight_wanted = true
		end
	end

	local main_bool = self._controller:get_input_bool("melee") or false
	local main_release = self._controller:get_input_released("melee")  or false
	local main_pressed = self._controller:get_input_pressed("melee")  or false

	local alt_bool = self._controller:get_input_bool("eatd_melee_butt") or false
	local alt_release = self._controller:get_input_released("eatd_melee_butt") or false
	local alt_pressed = self._controller:get_input_pressed("eatd_melee_butt") or false

	local main_melee = main_bool or main_release or main_pressed
	local alternate_melee = alt_bool or alt_release or alt_pressed

	if managers.blackmarket:equipped_melee_weapon() == "weapon" then
		input.btn_melee_press = main_pressed
		input.btn_melee_release = main_release
		input.btn_meleet_state = main_bool

		self:_update_melee_timers_bayonet( t, input )
	else
		if self._state_data.alternate_melee or alternate_melee then
			input.btn_melee_press = alt_pressed
			input.btn_melee_release = alt_release
			input.btn_meleet_state = alt_bool

			self:_update_melee_timers_bayonet( t, input )
		end

		if self._state_data.normal_melee or main_melee then
			input.btn_melee_press = main_pressed
			input.btn_melee_release = main_release
			input.btn_meleet_state = main_bool

			self:_update_melee_timers_normal( t, input )
		end
	end
end

function PlayerStandard:_start_action_melee_bayonet(t, input, instant)
	self._equipped_unit:base():tweak_data_anim_stop("fire")
	self:_interupt_action_reload(t)
	self:_interupt_action_steelsight(t)
	self:_interupt_action_running(t)
	self:_interupt_action_charging_weapon(t)

	self._state_data.melee_charge_wanted = nil
	self._state_data.meleeing = true
	self._state_data.melee_start_t = nil

	if instant then
		self:_do_action_melee(t, input)

		return
	end

	self:_stance_entered()

	local current_state_name = self._camera_unit:anim_state_machine():segment_state(self:get_animation("base"))
	local attack_allowed_expire_t = self._state_data.melee_data.attack_allowed_expire_t or 0.15
	self._state_data.melee_attack_allowed_t = t + (current_state_name ~= self:get_animation("melee_attack_state") and attack_allowed_expire_t or 0)
	local instant_hit = self._state_data.melee_data.instant

	if not instant_hit then
		self._ext_network:send("sync_melee_start")
	end
end

function PlayerStandard:_get_melee_charge_lerp_value_bayonet(t, offset)
	offset = offset or 0
	local max_charge_time = self._state_data.melee_data.stats.charge_time

	if not self._state_data.melee_start_t then
		return 0
	end

	return math.clamp((t - self._state_data.melee_start_t) - offset, 0, max_charge_time) / max_charge_time
end

function PlayerStandard:_do_action_melee_bayonet(t, input, skip_damage)
	self._state_data.meleeing = nil
	
	local melee_data = self._state_data.melee_data

	local instant_hit = melee_data.instant
	local pre_calc_hit_ray = melee_data.hit_pre_calculation
	local melee_damage_delay = melee_data.melee_damage_delay or 0
	melee_damage_delay = math.min(melee_damage_delay, melee_data.repeat_expire_t)

	self._state_data.melee_expire_t = t + melee_data.expire_t
	self._state_data.melee_repeat_expire_t = t + math.min(melee_data.repeat_expire_t, melee_data.expire_t)

	if not instant_hit and not skip_damage then
		self._state_data.melee_damage_delay_t = t + melee_damage_delay

		if pre_calc_hit_ray then
			self._state_data.melee_hit_ray = self:_calc_melee_hit_ray(t, 20) or true
		else
			self._state_data.melee_hit_ray = nil
		end
	end

	if instant_hit then
		managers.network:session():send_to_peers_synched("play_distance_interact_redirect", self._unit, "melee_bayonet")
	else
		self._ext_network:send("sync_melee_discharge")
	end

	if self._state_data.melee_charge_shake then
		self._ext_camera:shaker():stop(self._state_data.melee_charge_shake)

		self._state_data.melee_charge_shake = nil
	end

	self._melee_attack_var = 0

	local hit = skip_damage or self:_do_melee_damage(t, true)

	if hit then
		self._ext_camera:play_redirect( self:get_animation( melee_data.animation or "melee" ), melee_data.animation_multiplier or 1 )
	else
		self._ext_camera:play_redirect( self:get_animation( melee_data.miss_animation or "melee_miss" ), melee_data.miss_animation_multiplier or melee_data.animation_multiplier or 1 )
	end
end

local melee_vars = {
	"player_melee",
	"player_melee_var2"
}
function PlayerStandard:_do_melee_damage_bayonet(t, bayonet_melee, melee_hit_ray, melee_entry, hand_id)
	local bayonet_melee = self._state_data.bayonet_id and true
	local melee_entry = "weapon"

	local instant_hit = self._state_data.melee_data.instant
	local melee_damage_delay = self._state_data.melee_data.melee_damage_delay or 0
	local charge_lerp_value = instant_hit and 0 or self:_get_melee_charge_lerp_value(t, melee_damage_delay)

	self._ext_camera:play_shaker(melee_vars[math.random(#melee_vars)], math.max(0.3, charge_lerp_value))

	local sphere_cast_radius = 20
	local col_ray = nil

	if melee_hit_ray then
		col_ray = melee_hit_ray ~= true and melee_hit_ray or nil
	else
		col_ray = self:_calc_melee_hit_ray(t, sphere_cast_radius)
	end

	if col_ray and alive(col_ray.unit) then
		local damage, damage_effect = managers.blackmarket:equipped_melee_weapon_damage_info(charge_lerp_value)
		local damage_effect_mul = math.max(managers.player:upgrade_value("player", "melee_knockdown_mul", 1), managers.player:upgrade_value(self._equipped_unit:base():weapon_tweak_data().categories and self._equipped_unit:base():weapon_tweak_data().categories[1], "melee_knockdown_mul", 1))
		damage = damage * managers.player:get_melee_dmg_multiplier()
		damage_effect = damage_effect * damage_effect_mul
		col_ray.sphere_cast_radius = sphere_cast_radius
		local hit_unit = col_ray.unit

		if hit_unit:character_damage() then
			if bayonet_melee then
				self._unit:sound():play("fairbairn_hit_body", nil, false)
			else
				local hit_sfx = "hit_body"

				if hit_unit:character_damage() and hit_unit:character_damage().melee_hit_sfx then
					hit_sfx = hit_unit:character_damage():melee_hit_sfx()
				end

				self:_play_melee_sound(melee_entry, hit_sfx, self._melee_attack_var)
			end

			if not hit_unit:character_damage()._no_blood then
				managers.game_play_central:play_impact_flesh({col_ray = col_ray})
				managers.game_play_central:play_impact_sound_and_effects({
					no_decal = true,
					no_sound = true,
					col_ray = col_ray
				})
			end

			self._camera_unit:base():play_anim_melee_item("hit_body")
		elseif self._on_melee_restart_drill and hit_unit:base() and (hit_unit:base().is_drill or hit_unit:base().is_saw) then
			hit_unit:base():on_melee_hit(managers.network:session():local_peer():id())
		else
			if bayonet_melee then
				self._unit:sound():play("knife_hit_gen", nil, false)
			else
				self:_play_melee_sound(melee_entry, "hit_gen", self._melee_attack_var)
			end

			self._camera_unit:base():play_anim_melee_item("hit_gen")
			managers.game_play_central:play_impact_sound_and_effects({
				no_decal = true,
				no_sound = true,
				col_ray = col_ray,
				effect = Idstring("effects/payday2/particles/impacts/fallback_impact_pd2")
			})
		end

		local custom_data = nil

		if _G.IS_VR and hand_id then
			custom_data = {engine = hand_id == 1 and "right" or "left"}
		end

		managers.rumble:play("melee_hit", nil, nil, custom_data)
		managers.game_play_central:physics_push(col_ray)

		local character_unit, shield_knock = nil
		local can_shield_knock = managers.player:has_category_upgrade("player", "shield_knock")

		if can_shield_knock and hit_unit:in_slot(8) and alive(hit_unit:parent()) and not hit_unit:parent():character_damage():is_immune_to_shield_knockback() then
			shield_knock = true
			character_unit = hit_unit:parent()
		end

		character_unit = character_unit or hit_unit

		if character_unit:character_damage() and character_unit:character_damage().damage_melee then
			local dmg_multiplier = 1
			dmg_multiplier = not managers.enemy:is_civilian(character_unit) and not managers.groupai:state():is_enemy_special(character_unit) and dmg_multiplier * managers.player:upgrade_value("player", "non_special_melee_multiplier", 1) or dmg_multiplier * managers.player:upgrade_value("player", "melee_damage_multiplier", 1)
			dmg_multiplier = dmg_multiplier * managers.player:upgrade_value("player", "melee_" .. tostring(self._state_data.melee_data.stats.weapon_type) .. "_damage_multiplier", 1)

			if managers.player:has_category_upgrade("melee", "stacking_hit_damage_multiplier") then
				self._state_data.stacking_dmg_mul = self._state_data.stacking_dmg_mul or {}
				self._state_data.stacking_dmg_mul.melee = self._state_data.stacking_dmg_mul.melee or {
					nil,
					0
				}
				local stack = self._state_data.stacking_dmg_mul.melee

				if stack[1] and t < stack[1] then
					dmg_multiplier = dmg_multiplier * (1 + managers.player:upgrade_value("melee", "stacking_hit_damage_multiplier", 0) * stack[2])
				else
					stack[2] = 0
				end
			end

			local health_ratio = self._ext_damage:health_ratio()
			local damage_health_ratio = managers.player:get_damage_health_ratio(health_ratio, "melee")

			if damage_health_ratio > 0 then
				local damage_ratio = damage_health_ratio
				dmg_multiplier = dmg_multiplier * (1 + managers.player:upgrade_value("player", "melee_damage_health_ratio_multiplier", 0) * damage_ratio)
			end

			dmg_multiplier = dmg_multiplier * managers.player:temporary_upgrade_value("temporary", "berserker_damage_multiplier", 1)
			local target_dead = character_unit:character_damage().dead and not character_unit:character_damage():dead()
			local target_hostile = managers.enemy:is_enemy(character_unit) and not tweak_data.character[character_unit:base()._tweak_table].is_escort and character_unit:brain():is_hostile()
			local life_leach_available = managers.player:has_category_upgrade("temporary", "melee_life_leech") and not managers.player:has_activate_temporary_upgrade("temporary", "melee_life_leech")

			if target_dead and target_hostile and life_leach_available then
				managers.player:activate_temporary_upgrade("temporary", "melee_life_leech")
				self._unit:character_damage():restore_health(managers.player:temporary_upgrade_value("temporary", "melee_life_leech", 1))
			end

			local special_weapon = self._state_data.melee_data.special_weapon
			local action_data = {variant = "melee"}

			if special_weapon == "taser" then
				action_data.variant = "taser_tased"
			end

			if _G.IS_VR and melee_entry == "weapon" and not bayonet_melee then
				dmg_multiplier = 0.1
			end

			action_data.damage = shield_knock and 0 or damage * dmg_multiplier
			action_data.damage_effect = damage_effect
			action_data.attacker_unit = self._unit
			action_data.col_ray = col_ray

			if shield_knock then
				action_data.shield_knock = can_shield_knock
			end

			action_data.name_id = "weapon"
			action_data.charge_lerp_value = charge_lerp_value

			if managers.player:has_category_upgrade("melee", "stacking_hit_damage_multiplier") then
				self._state_data.stacking_dmg_mul = self._state_data.stacking_dmg_mul or {}
				self._state_data.stacking_dmg_mul.melee = self._state_data.stacking_dmg_mul.melee or {
					nil,
					0
				}
				local stack = self._state_data.stacking_dmg_mul.melee

				if character_unit:character_damage().dead and not character_unit:character_damage():dead() then
					stack[1] = t + managers.player:upgrade_value("melee", "stacking_hit_expire_t", 1)
					stack[2] = math.min(stack[2] + 1, tweak_data.upgrades.max_melee_weapon_dmg_mul_stacks or 5)
				else
					stack[1] = nil
					stack[2] = 0
				end
			end

			local defense_data = character_unit:character_damage():damage_melee(action_data)

			self:_check_melee_dot_damage(col_ray, defense_data, "weapon")
			self:_perform_sync_melee_damage(hit_unit, col_ray, action_data.damage)

			return defense_data
		else
			self:_perform_sync_melee_damage(hit_unit, col_ray, damage)
		end
	end

	if managers.player:has_category_upgrade("melee", "stacking_hit_damage_multiplier") then
		self._state_data.stacking_dmg_mul = self._state_data.stacking_dmg_mul or {}
		self._state_data.stacking_dmg_mul.melee = self._state_data.stacking_dmg_mul.melee or {
			nil,
			0
		}
		local stack = self._state_data.stacking_dmg_mul.melee
		stack[1] = nil
		stack[2] = 0
	end

	return col_ray
end

function PlayerStandard:_calc_melee_hit_ray_bayonet(t, sphere_cast_radius)
	local range = self._state_data.melee_data.stats.range or 175
	local from = self._unit:movement():m_head_pos()
	local to = from + self._unit:movement():m_head_rot():y() * range

	return self._unit:raycast("ray", from, to, "slot_mask", self._slotmask_bullet_impact_targets, "sphere_cast_radius", sphere_cast_radius, "ray_type", "body melee")
end

function PlayerStandard:_check_melee_dot_damage_bayonet(col_ray, defense_data, melee_entry)
	if not defense_data or defense_data.type == "death" then
		return
	end

	local dot_data = self._state_data.melee_data.dot_data

	if not dot_data then
		return
	end

	local data = managers.dot:create_dot_data(dot_data.type, dot_data.custom_data)
	local damage_class = CoreSerialize.string_to_classtable(data.damage_class)

	damage_class:start_dot_damage(col_ray, nil, data, "weapon")
end

local bayonet_variants = {
	"_start_action_melee",
	"_get_melee_charge_lerp_value",
	"_do_action_melee",
	"_calc_melee_hit_ray",
	"_do_melee_damage",
	"_check_melee_dot_damage"
}

for _, function_name in pairs(bayonet_variants) do
	PlayerStandard[function_name .. "_normal"] = PlayerStandard[function_name]
	PlayerStandard[function_name] = function( self, ... )
		if ( self._state_data.alternate_melee ) then
			return PlayerStandard[function_name .. "_bayonet"]( self, ... )
		end

		return PlayerStandard[function_name .. "_normal"]( self, ... )
	end
end

function PlayerStandard:_update_bayonet_id()
	if not self._state_data then return end

	self._state_data.bayonet_id = nil
	self._state_data.melee_data = nil
	
	local weapon = self._equipped_unit:base()

	local weapon_id = weapon:get_name_id()
	local bayonet_id = managers.blackmarket:actual_equipped_bayonet(weapon_id, self._equipped_unit:base():selection_index() == 2)

	if bayonet_id then
		self._state_data.bayonet_id = bayonet_id
		self._state_data.melee_data = managers.blackmarket:get_melee_data( bayonet_id, weapon._factory_id, weapon._blueprint )
	else
		self._state_data.melee_data = tweak_data.blackmarket.melee_weapons["weapon"]
	end
end

Hooks:PostHook( PlayerStandard, "enter", "BayonetFixes_enter", function( self )
	self:_update_bayonet_id()
end)

Hooks:PostHook( PlayerStandard, "_start_action_equip_weapon", "BayonetFixes__start_action_equip_weapon", function( self )
	self:_update_bayonet_id()
end)