Hooks:PostHook( PlayerStandard, "init", "ScopeOverlay_Init", function(self)
	self._state_data._overlay_hide_weapon = false

	self._state_data._sight_overlay = nil
	self._state_data._second_sight_overlay = nil
end)

Hooks:PostHook( PlayerStandard, "enter", "ScopeOverlay_StateEnter", function(self)
	self:_update_steelsight_overlay()
end)

Hooks:PostHook( PlayerStandard, "_start_action_equip_weapon", "ScopeOverlay_WeaponSwitch", function(self)
	self:_update_steelsight_overlay()
end)

Hooks:PostHook( PlayerStandard, "_steelsight_on_end", "ScopeOverlay_SteelsightOn", function(self)
	local weap_base = self._equipped_unit:base()

	if weap_base and weap_base:is_second_sight_on() then
		self:_set_scope_overlay( self._state_data._second_sight_overlay )
	else
		self:_set_scope_overlay( self._state_data._sight_overlay )
	end
end)

Hooks:PostHook( PlayerStandard, "_steelsight_off_start", "ScopeOverlay_SteelsightOff", function(self)
	self:_set_scope_overlay( nil )
end)

Hooks:PostHook( PlayerStandard, "_toggle_second_sight", "ScopeOverlay_ToggleSecondSight", function(self, second_sight_on)
	if self:in_steelsight() then
		if second_sight_on then
			self:_set_scope_overlay( self._state_data._second_sight_overlay )
		else
			self:_set_scope_overlay( self._state_data._sight_overlay )
		end
	end
end)

function PlayerStandard:_update_steelsight_overlay()
	self._state_data._sight_overlay = nil
	self._state_data._second_sight_overlay = nil

	self._state_data._overlay_hide_weapon = false

	local weapon = self._equipped_unit:base()

	for index, part_id in ipairs(weapon._blueprint) do
		local part_data = managers.weapon_factory:get_part_data_by_part_id_from_weapon(part_id, weapon._factory_id, weapon._blueprint)

		if part_data.scope_overlay then
			if ( part_data.type and ( part_data.type == "sight" ) ) then
				self._state_data._sight_overlay = part_data.scope_overlay
			elseif ( part_data.sub_type and ( part_data.sub_type == "second_sight" ) ) then
				self._state_data._second_sight_overlay = part_data.scope_overlay
			end
		end

		if part_data.scope_overlay_hide_weapon then
			if ( part_data.type and ( part_data.type == "sight" ) ) or ( part_data.sub_type and ( part_data.sub_type == "second_sight" ) ) then
				if part_data.scope_overlay_hide_weapon == true then -- This might seem odd but it stops string getting through.
					self._state_data._overlay_hide_weapon = true
				end
			end
		end
	end
end

function PlayerStandard:_set_scope_overlay( scope_overlay )
	if scope_overlay == nil then
		managers.hud._ScopeOverlayGUI:hide()
		return
	end

	managers.hud._ScopeOverlayGUI:set( scope_overlay, self._state_data._overlay_hide_weapon )
	managers.hud._ScopeOverlayGUI:show()
end